#!/usr/bin/env -S runhaskell --ghc-arg="-Wall" --ghc-arg="-i lib"

import GEODE.Metadata (PrimaryKey(..), readNamedTsv, relativePath)
import System.Directory (createDirectoryIfMissing, doesFileExist)
import System.Environment (getArgs)
import System.FilePath ((</>), takeDirectory)
import System.Posix.Files (createLink)
import System.Script (syntax, try)

link :: String -> FilePath -> FilePath -> PrimaryKey -> IO ()
link extension sourceRoot targetRoot article = do
  fileExists <- doesFileExist sourceFile
  if fileExists
  then do
    createDirectoryIfMissing True (takeDirectory targetFile)
    createLink sourceFile targetFile
  else
    putStrLn sourceFile
  where
    fileName = relativePath article extension
    sourceFile = sourceRoot </> fileName
    targetFile = targetRoot </> fileName

main :: IO ()
main = getArgs >>= run
  where
    run [files, extension, source, target] =
      try (readNamedTsv files) >>= mapM_ (link extension source target)
    run _ = syntax "FILES.tsv EXTENSION SOURCE_DIRECTORY TARGET_DIRECTORY"
